/*	Copyright (C) 2003-2015 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#include "terminal.pmh"

namespace fe
{

void TerminalDraw::drawPoints(const SpatialVector *vertex,
	const SpatialVector *normal,U32 vertices,
	BWORD multicolor,const Color *color,
	sp<DrawBufferI> spDrawBuffer)
{
	if(m_spSurfaceAccessibleI.isNull())
	{
		feLog("TerminalDraw::drawPoints SurfaceAccessibleI is NULL\n");
		return;
	}

	//* HACK
	return;

	//* TODO add to group
	sp<DrawMode> spDrawMode=drawMode();
	String group;
	if(spDrawMode.isValid())
	{
		group=spDrawMode->group();
	}

	sp<SurfaceAccessorI> spOutputPoint=m_spSurfaceAccessibleI->accessor(
			SurfaceAccessibleI::e_point,SurfaceAccessibleI::e_position);
	if(spOutputPoint.isNull())
	{
		feLog("TerminalDraw::drawPoints points not accessible\n");
		return;
	}

	sp<SurfaceAccessorI> spOutputNormal;
	if(normal)
	{
		spOutputNormal=m_spSurfaceAccessibleI->accessor(
				SurfaceAccessibleI::e_point,SurfaceAccessibleI::e_normal);
		if(spOutputNormal.isNull())
		{
			feLog("TerminalDraw::drawPoints normals not accessible\n");
			return;
		}
	}

	const I32 existingPointCount=spOutputPoint->count();

	spOutputPoint->append(vertices);

	for(U32 index=0;index<vertices;index++)
	{
		spOutputPoint->set(existingPointCount+index,vertex[index]);
	}

	if(spOutputNormal.isValid())
	{
		for(U32 index=0;index<vertices;index++)
		{
			spOutputNormal->set(existingPointCount+index,normal[index]);
		}
	}
}

void TerminalDraw::drawLines(const SpatialVector *vertex,
	const SpatialVector *normal,U32 vertices,
	StripMode strip,BWORD multicolor,
	const Color *color,const Vector3i *element,U32 elementCount,
	sp<DrawBufferI> spDrawBuffer)
{
	drawPolygons(vertex,normal,vertices,strip,multicolor,color,TRUE);
}

void TerminalDraw::drawCurve(const SpatialVector *vertex,
	const SpatialVector *normal,const Real *radius,U32 vertices,
	BWORD multicolor,const Color *color)
{
	if(radius==NULL)
	{
		drawLines(vertex,normal,vertices,DrawI::e_strip,
				multicolor,color,NULL,0,sp<DrawBufferI>(NULL));
		return;
	}

	//* append line segments with radius attribute

	if(m_spSurfaceAccessibleI.isNull())
	{
		feLog("TerminalDraw::drawCurve SurfaceAccessibleI is NULL\n");
		return;
	}

	//* TODO add to group
	sp<DrawMode> spDrawMode=drawMode();
	String group;
	if(spDrawMode.isValid())
	{
		group=spDrawMode->group();
	}

	sp<SurfaceAccessorI> spOutputPoint=m_spSurfaceAccessibleI->accessor(
			SurfaceAccessibleI::e_point,SurfaceAccessibleI::e_position);
	if(spOutputPoint.isNull())
	{
		feLog("TerminalDraw::drawCurve points not accessible\n");
		return;
	}

	sp<SurfaceAccessorI> spOutputNormal;
	if(normal)
	{
		spOutputNormal=m_spSurfaceAccessibleI->accessor(
				SurfaceAccessibleI::e_point,SurfaceAccessibleI::e_normal);
		if(spOutputNormal.isNull())
		{
			feLog("TerminalDraw::drawCurve normals not accessible\n");
			return;
		}
	}

	sp<SurfaceAccessorI> spOutputRadius;
	if(radius)
	{
		spOutputRadius=m_spSurfaceAccessibleI->accessor(
				SurfaceAccessibleI::e_point,"radius");
		if(spOutputRadius.isNull())
		{
			feLog("TerminalDraw::drawCurve radius not accessible\n");
			return;
		}
	}

	sp<SurfaceAccessorI> spOutputVertices=m_spSurfaceAccessibleI->accessor(
			SurfaceAccessibleI::e_primitive,SurfaceAccessibleI::e_vertices);
	if(spOutputVertices.isNull())
	{
		feLog("TerminalDraw::drawCurve vertices not accessible\n");
		return;
	}

	const I32 existingPointCount=spOutputPoint->count();
	const I32 primitiveIndex=spOutputVertices->count();

	//* TODO colors

	Array< Array<I32> > primVerts(1);

	Array<I32>& rVerts=primVerts[0];
	rVerts.resize(vertices);

	for(U32 index=0;index<vertices;index++)
	{
		rVerts[index]=existingPointCount+index;
	}

	spOutputVertices->append(primVerts);
	spOutputPoint->append(vertices);

	for(U32 index=0;index<vertices;index++)
	{
		spOutputPoint->set(existingPointCount+index,vertex[index]);
	}

	if(spOutputNormal.isValid())
	{
		for(U32 index=0;index<vertices;index++)
		{
			spOutputNormal->set(existingPointCount+index,normal[index]);
		}
	}

	if(spOutputRadius.isValid())
	{
		for(U32 index=0;index<vertices;index++)
		{
			spOutputRadius->set(existingPointCount+index,radius[index]);
		}
	}

	sp<SurfaceAccessorI> spOutputProperties=
		m_spSurfaceAccessibleI->accessor(
		SurfaceAccessibleI::e_primitive,SurfaceAccessibleI::e_properties);
	if(spOutputProperties.isValid())
	{
		spOutputProperties->set(primitiveIndex,
				SurfaceAccessibleI::e_openCurve,TRUE);
	}
}

void TerminalDraw::drawTriangles(const SpatialVector *vertex,
	const SpatialVector *normal,const Vector2f *texture,U32 vertices,
	StripMode strip,BWORD multicolor,const Color* color,
	sp<DrawBufferI> spDrawBuffer)
{
	//* TODO other strip modes

	if(strip==DrawI::e_discrete)
	{
		for(U32 m=0;m<vertices;m+=3)
		{
			if(color && multicolor)
			{
				drawPolygons(&vertex[m],&normal[m],3,DrawI::e_discrete,
						multicolor,&color[m],FALSE);
			}
			else
			{
				drawPolygons(&vertex[m],&normal[m],3,DrawI::e_discrete,
						multicolor,color,FALSE);
			}
		}
	}
	else if(strip==DrawI::e_strip)
	{
#if FALSE
		for(U32 m=0;m<vertices-2;m++)
		{
			if(color && multicolor)
			{
				drawPolygons(&vertex[m],&normal[m],3,DrawI::e_discrete,
						multicolor,&color[m],FALSE);
			}
			else
			{
				drawPolygons(&vertex[m],&normal[m],3,DrawI::e_discrete,
						multicolor,color,FALSE);
			}
		}
#else
		drawPolygons(vertex,normal,vertices,DrawI::e_strip,
				multicolor,color,FALSE);
#endif
	}
}

void TerminalDraw::drawPolygons(const SpatialVector *vertex,
	const SpatialVector *normal,U32 vertices,
	StripMode strip,BWORD multicolor,const Color *color,BWORD openPoly)
{
	if(m_spSurfaceAccessibleI.isNull())
	{
		feLog("TerminalDraw::drawPolygons SurfaceAccessibleI is NULL\n");
		return;
	}

//  feLog("TerminalDraw::drawTriangles"
//          " vertices %d strip %d multicolor %d color %p openPoly %d\n",
//          vertices, strip, multicolor, color, openPoly);

	//* TODO add to group
	sp<DrawMode> spDrawMode=drawMode();
	String group;
	if(spDrawMode.isValid())
	{
		group=spDrawMode->group();
	}

	sp<SurfaceAccessorI> spOutputPoint=m_spSurfaceAccessibleI->accessor(
			SurfaceAccessibleI::e_point,SurfaceAccessibleI::e_position);
	if(spOutputPoint.isNull())
	{
		feLog("TerminalDraw::drawPolygons points not accessible\n");
		return;
	}

	sp<SurfaceAccessorI> spOutputNormal;
	if(normal)
	{
		spOutputNormal=m_spSurfaceAccessibleI->accessor(
				SurfaceAccessibleI::e_point,SurfaceAccessibleI::e_normal);
		if(spOutputNormal.isNull())
		{
			feLog("TerminalDraw::drawPolygons normals not accessible\n");
			return;
		}
	}

	sp<SurfaceAccessorI> spOutputColor;
	if(color)
	{
		spOutputColor=m_spSurfaceAccessibleI->accessor(
				SurfaceAccessibleI::e_point,SurfaceAccessibleI::e_color);
		if(spOutputColor.isNull())
		{
			feLog("TerminalDraw::drawPolygons colors not accessible\n");
			return;
		}
	}

	sp<SurfaceAccessorI> spOutputVertices=m_spSurfaceAccessibleI->accessor(
			SurfaceAccessibleI::e_primitive,SurfaceAccessibleI::e_vertices);
	if(spOutputVertices.isNull())
	{
		feLog("TerminalDraw::drawPolygons vertices not accessible\n");
		return;
	}

	const I32 existingPointCount=spOutputPoint->count();
	const I32 primitiveIndex=spOutputVertices->count();

	Array< Array<I32> > primVerts(1);

	if(!openPoly && strip==DrawI::e_strip)
	{
		//* triangle strip

		const I32 addCount=vertices-2;
		I32 startIndex=existingPointCount;

		primVerts.resize(addCount);

		for(I32 addIndex=0;addIndex<addCount;addIndex++)
		{
			Array<I32>& rVerts=primVerts[addIndex];
			rVerts.resize(3);

			for(U32 index=0;index<3;index++)
			{
				rVerts[index]=startIndex+index;
			}

			startIndex++;
		}
	}
	else
	{
		Array<I32>& rVerts=primVerts[0];
		rVerts.resize(vertices);

		for(U32 index=0;index<vertices;index++)
		{
			rVerts[index]=existingPointCount+index;
		}
	}

	spOutputPoint->append(vertices);
	spOutputVertices->append(primVerts);

	for(U32 index=0;index<vertices;index++)
	{
		spOutputPoint->set(existingPointCount+index,vertex[index]);
	}

	if(spOutputNormal.isValid())
	{
		for(U32 index=0;index<vertices;index++)
		{
			spOutputNormal->set(existingPointCount+index,normal[index]);
		}
	}

	if(spOutputColor.isValid())
	{
		for(U32 index=0;index<vertices;index++)
		{
			spOutputColor->set(existingPointCount+index,
					color[multicolor? index: 0]);
		}
	}

	if(openPoly)
	{
		sp<SurfaceAccessorI> spOutputProperties=
			m_spSurfaceAccessibleI->accessor(
			SurfaceAccessibleI::e_primitive,SurfaceAccessibleI::e_properties);
		if(spOutputProperties.isValid())
		{
			spOutputProperties->set(primitiveIndex,
					SurfaceAccessibleI::e_openCurve,TRUE);
		}
	}
}

} // namespace
