/*	Copyright (C) 2003-2015 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#include "vegetation.pmh"

namespace fe
{

void SimPlant::handle(Record& record)
{
	if(!m_plantRV.scope().isValid())
	{
		sp<Scope> spScope=record.layout()->scope();
		m_plantRV.bind(spScope);
		m_seedRV.bind(spScope);
	}

	m_forestRV.bind(record);
	const F32 deltaT=m_forestRV.deltaT();

	if(deltaT==0.0f)
	{
		return;
	}

	sp<RecordGroup> spRG=m_forestRV.plants();
	FEASSERT(spRG.isValid());

	for(RecordGroup::iterator it=spRG->begin();it!=spRG->end();it++)
	{
		sp<RecordArray> spRA= *it;
		if(!m_plantRV.plantModel.check(spRA))
			continue;

		m_plantRV.bind(spRA);
		while(m_plantRV.step())
		{
			sp<PlantModelI> spPlantModelI=m_plantRV.plantModel();
			if(!spPlantModelI.isValid())
			{
				sp<Scope> spScope=
						registry()->master()->catalog()->catalogComponent(
						"Scope","PlantScope");

				const String path=
						registry()->master()->catalog()->catalog<String>(
						"path:media")+
						"/vegetation/"+m_plantRV.plantName()+".rg";

				sp<RecordGroup>	spRG=RecordView::loadRecordGroup(
						spScope,path);
				if(spRG.isNull())
				{
					feX("SimPlant::handle","failed to load \"%s\"",
							path.c_str());
					return;
				}

				RecordGroup::iterator it=spRG->begin();
				sp<RecordArray>& rspRA= *it;
				Record seed=rspRA->getRecord(0);

				const char* modelname=m_seedRV.modelname(seed).c_str();
				String componentName;
				componentName.sPrintf("PlantModelI.%s",modelname);
				spPlantModelI=registry()->create(componentName);
				if(!spPlantModelI.isValid())
				{
					feX("SimPlant::handle", "couldn't create plant model %s",
							modelname);
				}
				m_plantRV.plantModel()=spPlantModelI;

				spPlantModelI->generate(seed);
			}
			spPlantModelI->setWind(m_forestRV.windDirection());
			spPlantModelI->setCollider(m_forestRV.collider());
			spPlantModelI->setColliderTransform(m_forestRV.colliderTransform());
			spPlantModelI->setLocation(m_plantRV.location());
			spPlantModelI->update(deltaT);
			m_forestRV.effectForce()+=spPlantModelI->effectorForce();
		}
	}
}

} // namespace

